clear all
close all
% Initialization steps:
clc;    % Clear the command window.

workspace;  % Make sure the workspace panel is showing.
format long g;
format compact;

% Define a starting folder.
start_path = fullfile(matlabroot, '\toolbox');
if ~exist(start_path, 'dir')
	start_path = matlabroot;
end
% Ask user to confirm the folder, or change it.
uiwait(msgbox('Pick a starting folder on the next window that will come up.'));
topLevelFolder = uigetdir(start_path);

fprintf('The top level folder is "%s".\n', topLevelFolder);
% Specify the file pattern.
% Get ALL files using the pattern *.*
%
% Uncomment this if you want to get image files.
	% Add on JPG files.
 	filePattern = sprintf('%s/*.tiff', topLevelFolder);
 	allFileInfo = dir(filePattern);
    listOfFolderNames = unique({allFileInfo.folder});
    numberOfFolders = length(listOfFolderNames);
    [filepath,name,ext] = fileparts(topLevelFolder);
    figure_file = sprintf('%s_msdfig',name);
    mkdir(filepath,figure_file)
    %% Read in all images from the sorted folder in a cell
track_image = imageDatastore(topLevelFolder);
numImages = numel(track_image.Files);
%%
 n=1;                                          

   pa =1;
   y = cell(40,1);
   x = cell(40,1);
    msd_plateau =cell(6,1);
    msd_caged = cell(6,1);
    msd_trapped =cell(6,1);
for n =1:numImages
n

    image_set = track_image.Files{n};
    
    meta = imreadBFmeta(track_image.Files{n});
    zend = meta.zsize;
    xend = meta.width;
    yend = meta.height;
    tend = meta.nframes;
    
    %pre-allocate memory
    data = zeros(yend,xend);
    blurdata = data;
    
    dxdy = 0.16;%um/px scale
    dt = 0.051;%frame time in s
    
    noise_sz = 1;
    feat_size = 15;
    min_feat_size = 2;
    max_feat_size = 100;
    
    a = zeros(1,5);
  try
    for tstep = 100:tend
        tstep/tend*100;
        data = imread(image_set,tstep); %% read data by each frame
        
        blursize = 1;
        bldata = imgaussfilt(data,blursize);%% blur data
        %figure;
       % imshow(bldata);
        b = bpass2D_TA(bldata,noise_sz,feat_size+1);%% use a band pass filter
       % figure;
       % imshow(b);
        [BG EM] = multithresh(b(:),2); %% find a threshold; if it doesn't work set threshold manually
        BG = min(BG(:));
        b(b(:)<BG)=0;
        %figure;
       %imshow(b)
        pk = pkfnd(b,BG,feat_size); %% find local maxima
        centroids = cntrd(b,pk,feat_size,0); %% find centroid around local maxima
                     
        if size(centroids,2) > 0
            xs = centroids(:,1);
            ys = centroids(:,2);
            areas = centroids(:,4);
            intensities = centroids(:,3);
            
            %%%%%%% size exclusion %%%%
            
            if length(find(areas(:)>max_feat_size^2))>0
                xs([find(areas(:)<min_feat_size^2)' find(areas(:)>max_feat_size^2)'])=[];
                ys([find(areas(:)<min_feat_size^2)' find(areas(:)>max_feat_size^2)'])=[];
                intensities([find(areas(:)<min_feat_size^2)' find(areas(:)>max_feat_size^2)'])=[];
                areas([find(areas(:)<min_feat_size^2)' find(areas(:)>max_feat_size^2)'])=[];
                
            else
                xs(find(areas(:)<((min_feat_size^2))))=[];
                ys(find(areas(:)<((min_feat_size^2))))=[];
                intensities(find(areas(:)<((min_feat_size^2))))=[];
                areas(find(areas(:)<((min_feat_size^2))))=[];
            end
            
            %%%%%%%%%%%%%%% excluding close cells %%%%%%%%%%%
            temp = [xs ys];
            if size(temp,2)>0
                for step = 1:size(temp,1)
                    rtemp = sqrt(sum((temp(:,1:2)*dxdy-temp(step,1:2)*dxdy).^2,2));
                    xs(logical(logical(rtemp(:)<=3).*logical(rtemp(:)>0)))=NaN;
                    ys(logical(logical(rtemp(:)<=3).*logical(rtemp(:)>0)))=NaN;
                    intensities(logical(logical(rtemp(:)<=3).*logical(rtemp(:)>0)))=NaN;
                    areas(logical(logical(rtemp(:)<=3).*logical(rtemp(:)>0)))=NaN;
                end
            end
            xs(xs==NaN)=[];
            ys(ys==NaN)=[];
            intensities(intensities==NaN)=[];
            areas(areas==NaN)=[];
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            a = [a; xs ys  areas intensities repmat(tstep,[length(areas) 1])];
        end
    end
    catch ME
    fprintf(ME.message);
    continue; 
% %     pointsPrevious = points0Previous;
% %     featuresPrevious = features0Previous;
end 
    a(1,:)=[];
    %traj = figure;
   % plot(a(:,1),a(:,2),'rx')
     %   axis([0 220 0 220]);
    %csvwrite(['pre_tracks_',filename,'.csv'],a);

%%
count =0;
delta_r=[];
alpha_all = [0 0];
msd_all = [0 0];
alpha_msd = [0 0];


%%%%%%%%%%%%%%%%%%%%%%%%%%

max_feat_size = 15; % the max displacement particle between frames
motile_track = [0 0 0 0 0 0];

param.dim=2;% dimension
param.good=100;%set this keyword to eliminate all trajectories with
% ;             fewer than param.good valid positions.  This is useful
% ;             for eliminating very short, mostly 'lost' trajectories
param.quiet=1; %set this keyword to 1 if you don't want any text
param.mem=10; %this is the number of time steps that a particle can be
% ;             'lost' and then recovered again.  If the particle reappears
% ;             after this number of frames has elapsed, it will be
% ;             tracked as a new particle. The default setting is zero.
% ;             this is useful if particles occasionally 'drop out' of
% ;             the data.
maxdisp=max_feat_size;

if a ~=NaN
tracks = trackin(a,maxdisp,param);

if tracks == 0
    continue;
end
try
%% Mean square displacement
   
for particle = tracks(1,6):tracks(end,6)
    
    temptracks = tracks(tracks(:,6)==particle,:);
   % figure;
  %  hold on
   %plot(temptracks(:,1),temptracks(:,2))
   % hold off
   % correct for drift

   %figure;
   %plot(r(:,1),r(:,2))
    if length(temptracks(:,1))>=(param.good)
         x_part = temptracks(:,1);
         y_part = temptracks(:,2);
        v_drift = [mean(diff(x_part)),mean(diff(y_part))]/dt; % mean drift speed
         v = [diff((x_part)),diff(y_part)]/dt-v_drift;
       r=zeros(numel(x_part),2);
       
       for j=1:numel(x_part)-1
            r(j+1,:) =r(j,:)+v(j,:)*dt;
       end  
    for t = 0:length(r(:,1))-1
        dR_temp = r((1+t):end,1:2)*dxdy-r(1:(end-t),1:2)*dxdy;
        msd(t+1,1) = mean(squeeze(sum(dR_temp.^2,2))); %msd calculation
    end
     tau = dt*(0:length(msd)-1);
     y{pa} = msd(:,1);   
     x{pa} = dt*(0:length(msd)-1);
    pa = pa+1;
   % plot(tau,dR_1)
    % if max(msd(2:(param.good)))>(0.2^2)
           % max(msd(2:(param.good)))
          fp = figure;
           %fo=fit(tau(2:50)',msd(2:50)','poly1');
            loglog(tau(2:end-1),msd(2:end-1),'ob') %plot msd
            % hold on
             axis padded

             imagename = sprintf('%d_%d',n, particle);
             full_figname = fullfile(filepath,figure_file,imagename);

        %     print(imagename,'-dpng')
             savefig(fp,full_figname)
clc
clear MSD_plat
clear x_part y_part
   % end
 clear msd
    end
                close all
end
      n=n+1;         

catch ERR
%     %fprintf('oops on %d: %s',img_i,ERR.message);
    continue;
% %     pointsPrevious = points0Previous;
% %     featuresPrevious = features0Previous;
end
end
end
% %%

